@@ -62,6 +62,8 @@
 
 #' @export
 "length.magick-image" <- function(x){
+  if(magick_image_dead(x))
+    return(NULL)
   assert_image(x)
   magick_image_length(x)
 }
@@ -83,7 +85,7 @@
 "print.magick-image" <- function(x, info = TRUE, ...){
   img <- x
   viewer <- getOption("viewer")
-  if(length(img) && is.function(viewer)){
+  if(is.function(viewer) && !magick_image_dead(x) && length(img)){
     format <- tolower(image_info(img[1])$format)
     if(length(img) > 1 && format != "gif"){
       img <- image_animate(img, fps = 1)
