@@ -1,39 +1,46 @@
 #' Image Editing
 #'
-#' Read, write and join or combine images. All image functions are vectorized,
-#' meaning they operate either on a single frame or a series of frames (e.g. a
-#' collage, video, or animation).
-#' The \href{https://www.imagemagick.org/Magick++/STL.html}{Magick++ documentation}
-#' explains meaning of each function and parameter.
+#' Read, write and join or combine images.
+#' All image functions are vectorized, meaning they operate either on a single frame
+#' or a series of frames (e.g. a collage, video, or animation).
+#' Besides paths and URLs, the [image_read()] function supports all commonly used bitmap
+#' and raster types.
 #'
-#' Besides these functions also R-base functions such as \code{c()}, \code{[},
-#' \code{as.list()}, \code{as.raster()}, \code{rev}, \code{length}, and \code{print}
-#' can be used to work with image frames.
-#' See \link{transformations} for vectorized
-#' image manipulation functions such as cutting and applying effects.
+#' Besides functions
+#' above, all standard base vector methods such as \link{[}, \link{[[}, [c()], [as.list()],
+#' [as.raster()], [rev()], [length()], and [print()]  can be used with magick images.
 #'
-#' Some configurations of ImageMagick++ support reading SVG files but the rendering
-#' is not always very pleasing. Better results can be obtained by first rendering
-#' svg to a png using the \href{https://cran.r-project.org/package=rsvg}{rsvg package}.
+#' Use the standard \code{img[i]} syntax to extract a subset of the frames from an image.
+#' The \code{img[[i]]} method is used to extract a single frame as a bitmap object, i.e.
+#' a raw matrix with pixel values.
+#'
+#'
+#' X11 is required for `image_display()` which is only works on some platforms. A more
+#' portable method is `image_browse()` which opens the image in a browser. RStudio has
+#' an embedded viewer that does this automatically which is quite nice.
 #'
 #' @importFrom Rcpp sourceCpp
 #' @useDynLib magick
 #' @export
-#' @aliases magick imagemagick
 #' @family image
-#' @rdname edit
+#' @rdname editing
 #' @name editing
-#' @param path file path, URL, or raw array or \code{nativeRaster} with image data
-#' @param image object returned by \code{image_read}
+#' @param path a file, url, or raster object or bitmap array
+#' @param image magick image object returned by [image_read()] or [image_graph()]
 #' @param density resolution to render pdf or svg
-#' @param depth image depth. Must be 8 or 16
-#' @references Magick++ Image STL: \url{https://www.imagemagick.org/Magick++/STL.html}
 #' @examples
 #' # Download image from the web
 #' frink <- image_read("https://jeroen.github.io/images/frink.png")
 #' worldcup_frink <- image_fill(frink, "orange", "+100+200", 30000)
 #' image_write(worldcup_frink, "output.png")
 #'
+#' # extract raw bitmap array
+#' bitmap <- frink[[1]]
+#'
+#' # replace pixels with #FF69B4 ('hot pink') and convert back to image
+#' bitmap[,50:100, 50:100] <- as.raw(c(0xff, 0x69, 0xb4, 0xff))
+#' image_read(bitmap)
+#'
 #' # Plot to graphics device via legacy raster format
 #' raster <- as.raster(frink)
 #' par(ask=FALSE)
@@ -113,13 +120,14 @@ image_rsvg <- function(path, width = NULL, height = NULL){
 }
 
 #' @export
-#' @inheritParams transformations
-#' @rdname edit
+#' @inheritParams effects
+#' @rdname editing
 #' @param flatten should image be flattened before writing? This also replaces
 #' transparency with background color.
 #' @param quality number between 0 and 100 for jpeg quality. Defaults to 75.
+#' @param comment text string added to the image metadata for supported formats
 image_write <- function(image, path = NULL, format = NULL, quality = NULL,
-                        depth = NULL, density = NULL, flatten = FALSE){
+                        depth = NULL, density = NULL, comment = NULL, flatten = FALSE){
   assert_image(image)
   if(!length(image))
     warning("Writing image with 0 frames")
@@ -129,7 +137,8 @@ image_write <- function(image, path = NULL, format = NULL, quality = NULL,
   quality <- as.integer(quality)
   depth <- as.integer(depth)
   density <- as.character(density)
-  buf <- magick_image_write(image, format, quality, depth, density)
+  comment <- as.character(comment)
+  buf <- magick_image_write(image, format, quality, depth, density, comment)
   if(is.character(path)){
     writeBin(buf, path)
     return(invisible(path))
@@ -137,6 +146,26 @@ image_write <- function(image, path = NULL, format = NULL, quality = NULL,
   return(buf)
 }
 
+#' @export
+#' @rdname editing
+#' @param format output format such as `"png"`, `"jpeg"`, `"gif"`, `"rgb"` or `"rgba"`.
+#' @param depth color depth (either 8 or 16)
+#' @param antialias enable anti-aliasing for text and strokes
+#' @param type a magick [ImageType](https://www.imagemagick.org/Magick++/Enumerations.html#ImageType)
+#' classification for example `grayscale` to convert image to black/white
+#' @param colorspace string with a magick [ColorspaceType](https://www.imagemagick.org/Magick++/Enumerations.html#ColorspaceType)
+#' for example `"gray"`, `"rgb"` or `"cmyk"`
+image_convert <- function(image, format = NULL, type = NULL, colorspace = NULL, depth = NULL, antialias = NULL){
+  assert_image(image)
+  depth <- as.integer(depth)
+  antialias <- as.logical(antialias)
+  type <- as.character(type)
+  colorspace <- as.character(colorspace)
+  if(length(depth) && is.na(match(depth, c(8, 16))))
+    stop('depth must be 8 or 16 bit')
+  magick_image_format(image, toupper(format), type, colorspace, depth, antialias)
+}
+
 image_write_frame <- function(image, format = "rgb"){
   assert_image(image)
   magick_image_write_frame(image, format)
@@ -144,14 +173,14 @@ image_write_frame <- function(image, format = "rgb"){
 
 #' @export
 #' @param animate support animations in the X11 display
-#' @rdname edit
+#' @rdname editing
 image_display <- function(image, animate = TRUE){
   magick_image_display(image, animate)
 }
 
 #' @export
-#' @param browser argument passed to \link[utils:browseURL]{browseURL}
-#' @rdname edit
+#' @param browser argument passed to [browseURL][utils::browseURL]
+#' @rdname editing
 image_browse <- function(image, browser = getOption("browser")){
   ext <- ifelse(length(image), tolower(image_info(image[1])$format), "gif")
   tmp <- tempfile(fileext = paste0(".", ext))
@@ -160,138 +189,10 @@ image_browse <- function(image, browser = getOption("browser")){
 }
 
 #' @export
-#' @rdname edit
-#' @param stack place images top-to-bottom (TRUE) or left-to-right (FALSE)
-#' @examples
-#' # Create thumbnails from GIF
-#' banana <- image_read("https://jeroen.github.io/images/banana.gif")
-#' length(banana)
-#' image_average(banana)
-#' image_flatten(banana)
-#' image_append(banana)
-#' image_append(banana, stack = TRUE)
-#'
-#' # Append images together
-#' image_append(image_scale(c(image_append(banana[c(1,3)], stack = TRUE), frink)))
-image_append <- function(image, stack = FALSE){
-  assert_image(image)
-  magick_image_append(image, stack)
-}
-
-#' @export
-#' @rdname edit
-image_average <- function(image){
-  assert_image(image)
-  magick_image_average(image)
-}
-
-#' @export
-#' @rdname edit
-image_coalesce <- function(image){
-  assert_image(image)
-  magick_image_coalesce(image)
-}
-
-#' @export
-#' @inheritParams composite
-#' @rdname edit
-image_flatten <- function(image, operator = NULL){
-  assert_image(image)
-  operator <- as.character(operator)
-  magick_image_flatten(image, operator)
-}
-
-#' @export
-#' @rdname edit
-image_fft <- function(image){
-  if(!isTRUE(magick_config()$fftw))
-    stop("ImageMagick was configured without FFTW support. Reinstall with: brew install imagemagick --with-fftw")
-  assert_image(image)
-  magick_image_fft(image)
-}
-
-#' @export
-#' @rdname edit
-#' @param map reference image to map colors from
-#' @param dither set TRUE to enable dithering
-image_map <- function(image, map, dither = FALSE){
-  assert_image(image)
-  stopifnot(inherits(map, "magick-image"))
-  magick_image_map(image, map, dither)
-}
-
-#' @export
-#' @rdname edit
-image_montage <- function(image){
-  assert_image(image)
-  magick_image_montage(image)
-}
-
-#' @export
-#' @rdname edit
-#' @examples
-#' # Combine with another image
-#' logo <- image_read("https://www.r-project.org/logo/Rlogo.png")
-#' oldlogo <- image_read("https://developer.r-project.org/Logo/Rlogo-3.png")
-#'
-#' # Create morphing animation
-#' both <- image_scale(c(oldlogo, logo), "400")
-#' image_average(image_crop(both))
-#' image_animate(image_morph(both, 10))
-#' @param frames number of frames to use in output animation
-image_morph <- function(image, frames){
-  assert_image(image)
-  stopifnot(is.numeric(frames))
-  magick_image_morph(image, frames)
-}
-
-#' @export
-#' @rdname edit
-image_mosaic <- function(image, operator = NULL){
-  assert_image(image)
-  operator <- as.character(operator)
-  magick_image_mosaic(image, operator)
-}
-
-#' @export
-#' @rdname edit
-#' @param ... images or lists of images to be combined into a image
-#' @examples
-#' # Basic compositions
-#' image_composite(banana, image_scale(logo, "300"))
-#'
-#' # Break down and combine frames
-#' front <- image_scale(banana, "300")
-#' background <- image_background(image_scale(logo, "400"), 'white')
-#' frames <- image_apply(front, function(x){image_composite(background, x, offset = "+70+30")})
-#' image_animate(frames, fps = 10)
+#' @rdname editing
+#' @param ... several images or lists of images to be combined
 image_join <- function(...){
   x <- unlist(list(...))
   stopifnot(all(vapply(x, inherits, logical(1), "magick-image")))
   magick_image_join(x)
 }
-
-#' @export
-#' @rdname edit
-image_info <- function(image){
-  assert_image(image)
-  magick_image_info(image)
-}
-
-
-#' @export
-#' @rdname edit
-#' @param dispose frame disposal method. See
-#' \href{http://www.imagemagick.org/Usage/anim_basics/}{documentation}
-#' @param fps frames per second
-#' @param loop how many times to repeat the animation. Default is infinite.
-image_animate <- function(image, fps = 10, loop = 0, dispose = c("background", "previous", "none")){
-  assert_image(image)
-  stopifnot(is.numeric(fps))
-  stopifnot(is.numeric(loop))
-  if(100 %% fps)
-    stop("argument 'fps' must be a factor of 100")
-  delay <- as.integer(100/fps)
-  dispose <- match.arg(dispose)
-  magick_image_animate(image, delay, as.integer(loop), dispose)
-}
