@@ -26,6 +26,11 @@ Magick::Point Point(const char * str);
 #define myDrawableDashArray DrawableStrokeDashArray
 #define myMedianImage medianConvolveImage
 #define myFilterType FilterType
+#define myUndefinedMetric UndefinedErrorMetric
+#define myRedQ quantumRed
+#define myGreenQ quantumGreen
+#define myBlueQ quantumBlue
+#define myAlphaQ quantumAlpha
 #else
 #define container list
 #define Point Geom
@@ -34,6 +39,11 @@ Magick::Point Point(const char * str);
 #define myDrawableDashArray DrawableDashArray
 #define myMedianImage medianFilterImage
 #define myFilterType FilterTypes
+#define myUndefinedMetric UndefinedMetric
+#define myRedQ redQuantum
+#define myGreenQ greenQuantum
+#define myBlueQ blueQuantum
+#define myAlphaQ alphaQuantum
 #endif
 
 // Option parsers
